#!/bin/bash

#set -x
#set -e

function usage() {
    echo "Usage: `basename $0` path kstart kincrement kend repetitions coresetfactor projections"
    exit 1
}

# Paths
toolpath=./../tools/bin
algpath=./../algorithms/bin

# Settings
repair="0"                   #fixes up files if they dont match dimension

# better dont change something after this line unless you know what you do...

##################################################################################

# check for common mistakes
if [ -z "$1" -o ! -f "$1" -o $# -ne 7 ]; then
    usage
fi

if [ ! -d "$toolpath" ]; then
    echo "could not find directory $toolpath"
    exit 1
fi

if [ ! -d "$algpath" ]; then
    echo "could not find directory $algpath"
    exit 1
fi

# check for numerical values:
for i in `seq 2 1 7`; do
    pp=`eval echo '$'$i`
    if [ -z "$pp" ]; then
        echo "Param missing: $i"
		usage
        exit 1
    fi
    if ! [[ $pp =~ ^[0-9]+$ ]]; then
        echo "Param error: $pp"
		usage
        exit 1
    fi
done

# check for non-zero increments (would cause endless-loop)
if [ $3 -eq 0 ]; then
	echo "kinkremt should not be 0!"
	exit 1
fi


# extract working directory
wdir="`dirname $1`"

# extract centers
k=`seq $2 $3 $4`

# extract number of projections
p="$7"

# extract coreset factor
coresetfactor="$6"

# extract number of repetitions
r="$5"

# 1. validate CSV-file and transform to WSV-file
wsvpath="$1".wsv
foo="`\"${toolpath}/csv2wsv\" $repair \"$1\" \"${wsvpath}\"`"
if [ ! $? -eq 0 ]; then
  echo ${foo}
  exit 1
fi
n=${foo%% *}
d=${foo##$n }


# 2. create reportfile
date=`date`
datestamp=`date +"%F-Z%H:%M:%S"` 
report="${wdir}/report${datestamp}_`basename ${wsvpath}`.txt"

touch "$report"
cat > "$report" <<EOT
--- Clusteringreport ---

date: $date
call: `basename $0` $1 $2 $3 $4 $5 $6 $7

--- Fileinformation ---

input file: $1
number of points: $n
dimension: $d
projections: $p

EOT

# 3. Execute Bico algorithm and measure time
echo -e "\n----- Bico -----\n" >> "$report"

for j in $k; do
  m=$(($j * $coresetfactor))
  echo "--- coresetsize: $m" >> "$report"
  echo "-- expected number of centres: $j" >> "$report"
  echo "- runtime and memory" >> "$report"

  for l in `seq 1 1 $r`; do
   seed=`"${toolpath}/mt"` 
   echo "-random seed: $seed" >> "$report"
   echo "- runtime and memory:" >> "$report"
   /usr/bin/time -f "%U %M" "${algpath}/bico" "${wsvpath}.b" $n $j $d $m "${wdir}/bicocoreset${m}_${j}_${l}.txt" $p $seed > /dev/null 2>> "$report" 
   seed=`"${toolpath}/mt"`
   echo "-   2nd seed: $seed" >> "$report"
   /usr/bin/time -f "%U %M" "${algpath}/kmeans" "${wdir}/bicocoreset${m}_${j}_${l}.txt" $j $d "${wdir}/bicocenters${m}_${j}_${l}.txt" 0 $seed > /dev/null 2>> "$report"     
   echo "-------------------------------------------" >> "$report"
  done

  # calculate real number of centers
  echo "- real number of centres" >> "$report"
  for l in `seq 1 1 $r`; do
    foo=`wc -l "${wdir}/bicocenters${m}_${j}_${l}.txt"`
          realcenter=${foo%%"${wdir}/bicocenters${m}_${j}_${l}.txt"}           
    echo $realcenter >> "$report"
  done

  # calculate costs for bico
  echo -e "- costs" >> "$report"
  for l in `seq 1 1 $r`; do
    foo=`wc -l "${wdir}/bicocenters${m}_${j}_${l}.txt"`
    realcenter=${foo%%"${wdir}/bicocenters${m}_${j}_${l}.txt"}
    ${toolpath}/clustercosts "$wsvpath" "${wdir}/bicocenters${m}_${j}_${l}.txt" $d $n $realcenter >> "$report"
  done
  echo -e "\n" >> "$report"
done
echo -e "\n" >> "$report"

# 4. finish reportfile
echo -e "\n-------------------------
This run was performed on $HOSTNAME by $USER." >> "$report"

exit 0

################## 
