#!/bin/bash

#set -x
#set -e

function usage() {
    echo "Usage: `basename $0` path kstart kincrement kend birchmemory"
    exit 1
}

# Paths
toolpath=./../tools/bin
algpath=./../algorithms/bin

# Settings
repair="0"                   #fixes up files if they dont match dimension
cleanup="1"                  #true recommended
birchattributetype="int"     #use int or double here

# better dont change something after this line unless you know what you do...

##################################################################################

# check for common mistakes
if [ -z "$1" -o ! -f "$1" -o $# -ne 5 ]; then
    usage
fi

if [ ! -d "$toolpath" ]; then
    echo "could not find directory $toolpath"
    exit 1
fi

if [ ! -d "$algpath" ]; then
    echo "could not find directory $algpath"
    exit 1
fi

# check for numerical values:
for i in `seq 2 1 5`; do
    p=`eval echo '$'$i`
    if [ -z "$p" ]; then
        echo "Param missing: $i"
		usage
        exit 1
    fi
    if ! [[ $p =~ ^[0-9]+$ ]]; then
        echo "Param error: $p"
		usage
        exit 1
    fi
done

# check for non-zero increments (would cause endless-loop)
if [ $3 -eq 0 ]; then
	echo "kinkremt should not be 0!"
	exit 1
fi


# extract working directory
wdir="`dirname $1`"

# extract centers
k=`seq $2 $3 $4`

# extract algorithms
birchmemory="$5"

# 1. validate CSV-file and transform to WSV-file
wsvpath="$1".wsv
foo="`\"${toolpath}/csv2wsv\" $repair \"$1\" \"${wsvpath}\"`"
if [ ! $? -eq 0 ]; then
  echo ${foo}
  exit 1
fi
n=${foo%% *}
d=${foo##$n }


# 2. create reportfile
date=`date`
datestamp=`date +"%F-Z%H:%M:%S"` 
report="${wdir}/report${datestamp}_`basename ${wsvpath}`.txt"

touch "$report"
cat > "$report" <<EOT
--- Clusteringreport ---

date: $date
call: `basename $0` $1 $2 $3 $4 $5 $6 $7 $8 $9

--- Fileinformation ---

input file: $1
number of points: $n
dimension: $d

EOT


# 3. Exucte BIRCH and measure time
cp "${algpath}/birch" "${wdir}"
cd "${wdir}"
echo -e "\n----- BIRCH -----" >> "$report"

# create Proj-file
echo -n "$d " > "${wdir}/proj"
for i in `seq 1 1 $(($d - 1))`; do
  printf "x${i} " >> "${wdir}/proj"
done
echo "x${d}" >> "${wdir}/proj"  


# create Scheme-file
echo "type Generic-Schema ascii
comment //
separator ' '" > "${wdir}/scheme"
for i in `seq 1 1 $d`; do
  echo "attr x${i} ${birchattributetype}" >> "${wdir}/scheme"
done

for j in $k; do

  echo -e "expected number of centres: $j\n\n- runtime and memory" >> "$report"

# create Para-files
du=`du -b "${wsvpath}.b"`
bytesize=${du%%"${wsvpath}.b"}
bytesizepercent=$((${bytesize} / 100 * ${birchmemory}))
bytesizepercent2=$((${bytesizepercent} / 100 * ${birchmemory}))
cat > "${wdir}/para$j" <<EOT
0
$bytesizepercent
$bytesizepercent2
$bytesizepercent2
$bytesizepercent2

0
EOT

for i in `seq 1 1 $(($d - 1))`; do
  echo -n "1 " >> "${wdir}/para$j"
done
echo "1" >> "${wdir}/para$j"

for i in `seq 1 1 $(($d - 1))`; do
  echo -n "0 " >> "${wdir}/para$j"
done
echo -e "0\n" >> "${wdir}/para$j"

if [ $d -lt 50 ]; then
    echo -e "1024\n" >> "${wdir}/para$j"
else
    echo -e "2048\n" >> "${wdir}/para$j"
fi

cat >> "${wdir}/para$j" <<EOT
4
0
0
0
4
0.25

2000

0
0
EOT

for i in `seq 1 1 $(($d - 1))`; do
  echo -n "100 " >> "${wdir}/para$j"
done
cat >> "${wdir}/para$j" <<EOT
100

$j
0
0
1
2
0
1
0
1
EOT


# execute BIRCH
cd ${wdir}
	/usr/bin/time -f "%U %M" ${wdir}/birch para$j scheme proj `basename ${wsvpath}`.b > /dev/null 2>> "$report"

if [ $? -eq 0 ]; then
  # extract centers
  cat "${wdir}/para$j+scheme+proj+`basename ${wsvpath}`.b-0-cluster" | sed -n -e 's/^[^[:space:]]*[[:space:]]\+\(.*\)\([[:space:]]\+.\+\)\{2\}$/1 \1/; T; s/[[:space:]]\+/ /gp' > ${wdir}/birchcenters${j}.txt

  # determine exact number of centers
  echo "- real number of centres" >> "$report"
  foo=`wc -l "${wdir}/birchcenters${j}.txt"`
  realcenter=${foo%%"${wdir}/birchcenters${j}.txt"}           
  echo $realcenter >> "$report"
  
  # calcuate costs for BIRCH
  echo -e "\n- costs" >> "$report"
  foo=`wc -l "${wdir}/birchcenters${j}.txt"`
  realcenter=${foo%%"${wdir}/birchcenters${j}.txt"}     
  ${toolpath}/clustercosts "$wsvpath" "${wdir}/birchcenters${j}.txt" $d $n $realcenter >> "$report" 

  # remove some temporary files created by BIRCH
  rm "${wdir}/Tmp-cluster"
  rm "${wdir}/para$j+scheme+proj+`basename ${wsvpath}`.b-0-cluster"
  rm "${wdir}/para$j+scheme+proj+`basename ${wsvpath}`.b-log"
else
  echo "BIRCH failed!" >> "$report"
fi

done  # for j in $k

# 4. finish reportfile
echo -e "\n-------------------------
This run was performed on $HOSTNAME by $USER." >> "$report"

exit 0

################## 
