#!/bin/bash

#set -x
#set -e

function usage() {
    echo "Usage: `basename $0` pathToMacQueenFormat pathToNormalFormat kstart kincrement kend numberofpoints dimension"
    exit 1
}

# Paths
toolpath=./../tools/bin
algpath=./../algorithms/bin

# Settings
repair="0"                   #fixes up files if they dont match dimension

# better dont change something after this line unless you know what you do...

##################################################################################

# check for common mistakes
if [ -z "$1" -o ! -f "$1" -o $# -ne 7 ]; then
    usage
fi

if [ ! -d "$toolpath" ]; then
    echo "could not find directory $toolpath"
    exit 1
fi

if [ ! -d "$algpath" ]; then
    echo "could not find directory $algpath"
    exit 1
fi

# check for numerical values:
for i in `seq 3 1 7`; do
    p=`eval echo '$'$i`
    if [ -z "$p" ]; then
        echo "Param missing: $i"
		usage
        exit 1
    fi
    if ! [[ $p =~ ^[0-9]+$ ]]; then
        echo "Param error: $p"
		usage
        exit 1
    fi
done

# check for non-zero increments (would cause endless-loop)
if [ $3 -eq 0 ]; then
	echo "kinkremt should not be 0!"
	exit 1
fi


# extract working directory
wdir="`dirname $1`"

# extract centers
k=`seq $3 $4 $5`

# extract number of repetitions
r=1

# extract number of points
n="$6"

# extract dimension
d="$7"

# 2. create reportfile
date=`date`
datestamp=`date +"%F-Z%H:%M:%S"` 
report="${wdir}/report${datestamp}_`basename ${wdir}`.txt"

touch "$report"
cat > "$report" <<EOT
--- Clusteringreport ---

date: $date
call: `basename $0` $1 $2 $3 $4 $5 $6 $7

--- Fileinformation ---

input file: $1
number of points: $n
dimension: $d

(number and dimension were given as parameters)

EOT

pfaddatei="${wdir}/temppfaddatei"
touch "$pfaddatei"
cat > "$pfaddatei" <<EOT
$1
EOT

errorout="${wdir}/macQueen_errorout_${datestamp}_`basename ${wdir}`.txt"
touch "$errorout"

# 3. Execute streamKM++ algorithm and measure time
echo -e "----- MacQueen -------\n\n" >> "$report"

for j in $k; do
  echo "-- expected number of centres: $j" >> "$report"
  echo "--------------------------------------" >> "$report"
  echo "------------- runtime and memory -----" >> "$report"

  for l in `seq 1 1 $r`; do
    echo "- runtime and memory:" >> "$report"
    /usr/bin/time -f "%U %M" "${algpath}/macqueen" -m 1 $d $j "${pfaddatei}" > "${wdir}/macQueenOutfile_${j}_${l}.txt" 2>> "$errorout" 
    tail -n 1 "$errorout" >> "$report"
    echo "---------------------------------------------------" >> "$report"
  done

# Extract centers from reportfile
  
#  for j in $k; do
      for l in  `seq 1 1 $r`; do
	  "${toolpath}/output_queen" "${wdir}/macQueenOutfile_${j}_${l}.txt" "${wdir}/macQueenCenters_${j}_${l}.txt"
      done
#  done

# calculate real number of centers and costs
# number of centers
  echo "- real number of centres" >> "$report"
  for l in `seq 1 1 $r`; do
    foo=`wc -l "${wdir}/macQueenCenters_${j}_${l}.txt"`
          realcenter=${foo%%"${wdir}/macQueenCenters_${j}_${l}.txt"}           
    echo $realcenter >> "$report"
  done

# calculate costs for MacQueen
  echo -e "- costs" >> "$report"
  for l in `seq 1 1 $r`; do
          foo=`wc -l "${wdir}/macQueenCenters_${j}_${l}.txt"`
          realcenter=${foo%%"${wdir}/macQueenCenters_${j}_${l}.txt"}
    ${toolpath}/clustercosts_mq "${wdir}/macQueenCenters_${j}_${l}.txt" $2 $realcenter $d >> "$report"
  done
  echo -e "\n" >> "$report"
done
echo -e "\n" >> "$report"

# 4. finish reportfile
echo -e "\n-------------------------
This run was performed on $HOSTNAME by $USER." >> "$report"

exit 0

################## 
